package com.persagy.apm.diagnose.service;

import com.persagy.apm.diagnose.maintenance.service.IProjectDataRecordService;
import com.persagy.apm.diagnose.service.dto.ProjectDTO;
import com.persagy.apm.diagnose.utils.CommonUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.time.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Future;

/**
 * 设备维保数据 计算与发送
 */
@Slf4j
@Service
public class MaintenanceDataService {

    @Autowired
    private SaasWebServiceImpl saasWebServiceImpl;
    @Autowired
    private IProjectDataRecordService projectDataRecordService;

    /**
     * 计算 数据
     */
    public void computeMaintenanceData() throws Exception {

        //查询项目列表
        List<ProjectDTO> projectDTOList = saasWebServiceImpl.queryProjectDTOList(null);
        if (CollectionUtils.isEmpty(projectDTOList)) {
            return;
        }
        //获取当日日期
        Date todayDate = DateUtils.truncate(new Date(), Calendar.DATE);
//        projectDataRecordService.computeMaintenanceData(todayDate,"Pj1101060001", null);
        List<Future<String>> futureList = new ArrayList<>();
        CountDownLatch latch = new CountDownLatch(projectDTOList.size());
        for (ProjectDTO projectDTO : projectDTOList) {
            futureList.add(projectDataRecordService.computeMaintenanceData(todayDate,projectDTO.getProjectId(), latch));
        }
        //等待所有任务执行完
        latch.await();
        futureList.forEach(future -> {
            try {
                log.info(future.get());
            } catch (Exception e) {
                log.error("计算项目维保日维度数据错误:" + CommonUtils.getExceptionStackTrace(e));
            }
        });
    }

    /**
     * 发送数据
     */
    public void sendMaintenanceData() throws Exception {
        //查询项目列表
        List<ProjectDTO> projectDTOList = saasWebServiceImpl.queryProjectDTOList(null);
        if (CollectionUtils.isEmpty(projectDTOList)) {
            return;
        }
        //获取当日日期
        Date todayDate = DateUtils.truncate(new Date(), Calendar.DATE);
//        projectDataRecordService.sendMaintenanceData(todayDate,"Pj1101060001",null);
        List<Future<String>> futureList = new ArrayList<>();
        CountDownLatch latch = new CountDownLatch(projectDTOList.size());
        for (ProjectDTO projectDTO : projectDTOList) {
            futureList.add(projectDataRecordService.sendMaintenanceData(todayDate,projectDTO.getProjectId(),latch));
        }
        //等待所有任务执行完
        latch.await();
        futureList.forEach(future -> {
            try {
                log.info(future.get());
            } catch (Exception e) {
                log.error("发送指标数据错误:" + CommonUtils.getExceptionStackTrace(e));
            }
        });
    }

    /**
     * 同步即将报废设备
     */
    public void syncCheckGoingDestroyEquip() throws Exception {
//        projectDataRecordService.syncGoingDestroyEquip("Pj1101060001",null);
        //查询项目列表
        List<ProjectDTO> projectDTOList = saasWebServiceImpl.queryProjectDTOList(null);
        if (CollectionUtils.isEmpty(projectDTOList)) {
            return;
        }
        List<Future<String>> futureList = new ArrayList<>();
        CountDownLatch latch = new CountDownLatch(projectDTOList.size());
        for (ProjectDTO projectDTO : projectDTOList) {
            futureList.add(projectDataRecordService.syncGoingDestroyEquip(projectDTO.getProjectId(),latch));
        }
        //等待所有任务执行完
        latch.await();
        futureList.forEach(future -> {
            try {
                log.info(future.get());
            } catch (Exception e) {
                log.error("同步检查即将报废设备错误:" + CommonUtils.getExceptionStackTrace(e));
            }
        });
    }

}
